package org.htl.leonding.digitalsignage.wayfinder.entities;

import com.fasterxml.jackson.databind.annotation.JsonAppend;
import com.google.gson.annotations.Expose;
import com.oracle.webservices.internal.api.message.PropertySet;
import javafx.scene.paint.Color;
import javafx.scene.shape.Line;
import javafx.scene.text.Font;
import javafx.scene.text.FontWeight;
import javafx.scene.text.Text;
import org.htl.leonding.digitalsignage.wayfinder.interfaces.IObserver;
import org.neo4j.ogm.annotation.*;

import java.text.DecimalFormat;

/**
 * Created by Cristopher on 10/12/2015.
 */
@RelationshipEntity(type = "CONNECTS_TO")
public class ConnectRelation extends Line{

    @Expose
    @GraphId
    Long relationId;

    @Expose
    @Property(name = "length")
    private double length;

    @Expose
    boolean bidirectional;

    @Property(name = "startX")
    private double startXCoordinate;
    @Property(name = "startY")
    private double startYCoordinate;

    @Property(name = "endX")
    private double endXCoordinate;
    @Property(name = "endY")
    private double endYCoordindate;

    @Expose
    @Property
    private boolean dateRestricted;

    @Expose
    @Property
    private String dateFrom;

    @Expose
    @Property
    private String dateTo;

    @Expose
    @Property
    private boolean timeRestricted;

    @Expose
    @Property
    private String timeFrom;

    @Expose
    @Property
    private String timeTo;

    @Expose
    @Property
    private boolean disabledFriendly;

    @Property
    private boolean automaticLength = false;


    @Expose
    @StartNode
    MapNode startNode;

    @Expose
    @EndNode
    MapNode endNode;

    public Text getText() {
        return text;
    }

    public void setText(Text text) {
        this.text = text;
    }

    @Transient
    Text text;


    public ConnectRelation(){
        super();
        this.setStrokeWidth(2);
        this.setStroke(Color.BLUE);

        text = new Text();
    }

    public ConnectRelation(MapNode first, MapNode second) {
        this.startNode = first;
        this.endNode = second;
        this.setStrokeWidth(2);
        this.setStroke(Color.BLUE);
        updateStartAndEndAndText();

        setTimeFrom("");
        setTimeTo("");

        setDateFrom("");
        setDateTo("");

        disabledFriendly = true;
        bidirectional = true;

        text = new Text();
    }

    /**
     *
     * @param width
     * @param height
     */
    public void updateStartAndEndAndText(double width, double height){
        updateStartAndEnd();
        if(this.isAutomaticLength()){
            calcAutomaticLength(width, height);
        }
        updateText();
    }

    private void calcAutomaticLength(double width, double height){

        if(width <= 0 || height <= 0){
            return;
        }

        double origWidth = 1280;
        double origHeight = 720;

        double widthMultiplicator = width/origWidth;
        double heightMultiplicator = height/origHeight;

        double startX = this.getStartX()*widthMultiplicator;
        double endX = this.getEndX()*widthMultiplicator;
        double startY = this.getStartY()*heightMultiplicator;
        double endY = this.getEndY()*heightMultiplicator;

        this.length = Math.sqrt(Math.pow(startX-endX,2)+Math.pow(startY-endY,2));
    }

    private void updateStartAndEndAndText() {
        updateStartAndEnd();
        updateText();
    }

    private void updateStartAndEnd(){
        this.setStartX(startNode.getCenterX());
        this.setStartY(startNode.getCenterY());

        this.setEndX(endNode.getCenterX());
        this.setEndY(endNode.getCenterY());

        this.setStartXCoordinate(startNode.getCenterX());
        this.setStartYCoordinate(startNode.getCenterY());

        this.setEndXCoordinate(endNode.getCenterX());
        this.setEndYCoordindate(endNode.getCenterY());
    }

    private void updateText(){
        if(text == null){
            text = new Text();
        }

        DecimalFormat df = new DecimalFormat("#0.0");
        text.setText(df.format(length).toString());

        text.setX(((getStartX()+getEndX())/2)-(text.getLayoutBounds().getWidth()/2));
        text.setY(((getStartY()+getEndY())/2)-4);

        text.setFont(Font.font("Verdana", FontWeight.BOLD, 16f));

    }

    public boolean isDisabledFriendly() {
        return disabledFriendly;
    }

    public void setDisabledFriendly(boolean disabledFriendly) {
        this.disabledFriendly = disabledFriendly;
    }

    public double getLength() {
        return length;
    }

    public void setLength(double length) {
        this.length = length;
        text.setText(String.valueOf(length));
    }

    public MapNode getStartNode() {
        return startNode;
    }

    public void setStartNode(MapNode startNode) {
        this.startNode = startNode;
    }

    public boolean isBidirectional() {
        return bidirectional;
    }

    public void setBidirectional(boolean bidirectional) {
        this.bidirectional = bidirectional;
    }

    public Long getRelationId() {
        return relationId;
    }

    public void setRelationId(Long relationId) {
        this.relationId = relationId;
    }

    public MapNode getEndNode() {
        return endNode;
    }

    public void setEndNode(MapNode endNode) {
        this.endNode = endNode;
    }

    public double getStartXCoordinate() {
        return startXCoordinate;
    }

    public void setStartXCoordinate(double startXCoordinate) {
        this.startXCoordinate = startXCoordinate;
    }

    public double getStartYCoordinate() {
        return startYCoordinate;
    }

    public void setStartYCoordinate(double startYCoordinate) {
        this.startYCoordinate = startYCoordinate;
    }

    public double getEndXCoordinate() {
        return endXCoordinate;
    }

    public void setEndXCoordinate(double endXCoordinate) {
        this.endXCoordinate = endXCoordinate;
    }

    public double getEndYCoordindate() {
        return endYCoordindate;
    }

    public void setEndYCoordindate(double endYCoordindate) {
        this.endYCoordindate = endYCoordindate;
    }

    public boolean isTimeRestricted() {
        return timeRestricted;
    }

    public void setTimeRestricted(boolean timeRestricted) {
        this.timeRestricted = timeRestricted;
        if (!timeRestricted) {
            setTimeTo("");
            setTimeFrom("");
        }
    }

    public boolean isDateRestricted() {
        return dateRestricted;
    }

    public void setDateRestricted(boolean dateRestricted) {
        this.dateRestricted = dateRestricted;
        if (!dateRestricted) {
            setDateFrom("");
            setDateTo("");
        }
    }

    public String getDateFrom() {
        return dateFrom;
    }

    public void setDateFrom(String dateFrom) {
        this.dateFrom = dateFrom;
    }

    public String getDateTo() {
        return dateTo;
    }

    public void setDateTo(String dateTo) {
        this.dateTo = dateTo;
    }

    public String getTimeFrom() {
        return timeFrom;
    }

    public void setTimeFrom(String timeFrom) {
        String[] time = timeFrom.split(":");

        if (!timeFrom.isEmpty()) {
            if (time[0].length() < 2) {
                time[0] = "0" + time[0];
            }

            if (time[1].length() < 2) {
                time[1] = "0" + time[0];
            }
            this.timeFrom = time[0] + ":" + time[1];
        } else {
            this.timeFrom = timeFrom;
        }
    }

    public String getTimeTo() {
        return timeTo;
    }

    public void setTimeTo(String timeTo) {
        String[] time = timeTo.split(":");

        if (!timeTo.isEmpty()) {
            if (time[0].length() < 2) {
                time[0] = "0" + time[0];
            }

            if (time[1].length() < 2) {
                time[1] = "0" + time[0];
            }


            this.timeTo = time[0] + ":" + time[1];
        } else {
            this.timeTo = timeTo;
        }
    }

    public boolean isAutomaticLength() {
        return automaticLength;
    }

    public void setAutomaticLength(boolean automaticLength) {
        this.automaticLength = automaticLength;
    }

    @Override
    public String toString() {
        return "ID: " + this.getRelationId() + "Start: " + this.getStartNode() + " End: " + this.getEndNode() + " " + super.toString();
    }
}
