package org.htl.leonding.digitalsignage.wayfinder.entities;

import com.fasterxml.jackson.databind.annotation.JsonAppend;
import com.google.gson.annotations.Expose;
import javafx.scene.paint.Color;
import javafx.scene.shape.Circle;
import org.neo4j.ogm.annotation.*;

import java.util.ArrayList;
import java.util.List;
import org.htl.leonding.digitalsignage.wayfinder.models.MapNodeType;

/**
 * Created by Cristopher on 08/12/2015.
 */
@NodeEntity
public abstract class MapNode extends Circle {

    protected final static double DEFAULT_RADIUS = 8;

    @Expose
    @GraphId
    private Long nodeId;

    @Expose
    @Property
    protected String isA;


    @Expose
    @Property
    private String name;

    public Floor getFloor() {
        return floor;
    }

    public void setFloor(Floor floor) {
        this.floor = floor;
    }

    @Relationship(type="HAS_NODE", direction = Relationship.INCOMING)
    private Floor floor;


    @Relationship(type = "CONNECTS_TO",direction = Relationship.UNDIRECTED)
    private List<ConnectRelation> relations;

    @Expose
    @Property(name = "x")
    private double xCoordinate;

    @Expose
    @Property(name = "y")
    private double yCoordinate;

    public Long getNodeId() {
        return nodeId;
    }

    public void setNodeId(Long nodeId) {
        this.nodeId = nodeId;
    }

    public ConnectRelation connectToNode(MapNode otherNode) {
        ConnectRelation relation = new ConnectRelation(this, otherNode);
        this.getRelations().add(relation);
        otherNode.getRelations().add(relation);
        return relation;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public void setXCoordinate(double x) {
        this.xCoordinate = x;
        this.setCenterX(x);
    }

    public void setYCoordinate(double y) {
        this.yCoordinate = y;
        this.setCenterY(y);
    }

    public double getXCoordinate() {
        return xCoordinate;
    }

    public double getYCoordinate() {
        return yCoordinate;
    }

    public List<ConnectRelation> getRelations() {
        if (relations == null) {
            relations = new ArrayList<>();
        }
        return relations;
    }

    public void setRelations(List<ConnectRelation> relations) {
        this.relations = relations;
    }

    public static double getDefaultRadius() {
        return DEFAULT_RADIUS;
    }

    public MapNode() {
        super();
        this.relations = new ArrayList<>();
    }

    public MapNode(Color color, double radius) {
        super();
        this.setFill(color);
        this.setRadius(radius);
        this.relations = new ArrayList<ConnectRelation>();
    }

    public void notifyObservers() {
        getRelations().forEach(r -> r.updateStartAndEndAndText(floor.getFloorWidth(),floor.getFloorHeight()));
    }

    public String getIsA() {
        return isA;
    }

    public void setIsA(String isA) {
        this.isA = isA;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || nodeId == null || getClass() != o.getClass()) return false;

        MapNode node = (MapNode) o;

        if (!nodeId.equals(node.nodeId)) return false;

        return true;
    }

    @Override
    public int hashCode() {
        return (nodeId == null) ? -1 : nodeId.hashCode();
    }

    @Override
    public String toString() {
        return "id: "+ this.nodeId+ " XCoordinate: " + this.getXCoordinate() + " YCoordinate: " + this.yCoordinate + " " + super.toString();

    }

}