/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.htl.leonding.digitalsignage.wayfinder.models;


import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.reflect.TypeToken;
import com.sun.org.apache.xerces.internal.impl.dv.util.Base64;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.scene.control.Alert;
import javafx.scene.control.ButtonType;
import javafx.scene.effect.DropShadow;
import javafx.scene.paint.Color;
import javafx.stage.DirectoryChooser;
import javafx.stage.FileChooser;
import javafx.stage.Stage;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.htl.leonding.digitalsignage.wayfinder.serialization.ConnectRelationSerializer;
import org.htl.leonding.digitalsignage.wayfinder.serialization.MapNodeDeserializer;
import org.htl.leonding.digitalsignage.wayfinder.entities.*;
import org.htl.leonding.digitalsignage.wayfinder.repositories.DatabaseRepository;
import org.htl.leonding.digitalsignage.wayfinder.repositories.RemoveFloorFromDbThread;
import org.htl.leonding.digitalsignage.wayfinder.repositories.RemoveFromDbThread;
import org.htl.leonding.digitalsignage.wayfinder.repositories.SaveToDbThread;
import org.neo4j.ogm.json.JSONArray;
import org.neo4j.ogm.json.JSONException;
import org.neo4j.ogm.json.JSONObject;
import org.neo4j.ogm.session.Session;

import javax.annotation.PostConstruct;
import java.io.*;
import java.lang.reflect.Type;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.*;
import java.util.logging.Level;
import java.util.logging.Logger;

import static java.nio.file.StandardOpenOption.APPEND;
import static java.nio.file.StandardOpenOption.CREATE;

/**
 * @author Cristopher
 */
public class EditorModel extends Observable {

    private MapNode currentNode;
    private ConnectRelation currentRelation;
    private Floor currentFloor;

    private DropShadow highlight;

    //private ArrayList<Floor> floorList;
    private ObservableList<Floor> floorList;
    private List<ConnectRelation> relationList;


    boolean reloadFloor;

    boolean reloadListView;

    boolean batchRemove = false;

    Session session;

    Timer updateNodesTimer;

    @PostConstruct
    public void init() throws IOException {
        highlight = new DropShadow(20, Color.BLUE);
        highlight.setWidth(20);
        highlight.setHeight(20);
        session = DatabaseRepository.getInstance().getSession();
        currentFloor = null;
        currentRelation = null;
        currentNode = null;

        reloadFloor = false;
        reloadListView = false;
        //DatabaseRepository.getInstance().fillDb();
        relationList = new LinkedList<>();
        loadNodes();
    }

    public void resetUpdaters(){
        reloadListView = false;
        reloadFloor = false;
    }

    public void loadNodes() throws IOException {
        //Collection<MapNode> mapNodes = session.loadAll(MapNode.class, 1);
        floorList = FXCollections.observableArrayList(session.loadAll(Floor.class, 2));
        //LinkedList<MapNode> mapNodeList = new LinkedList<>();
        floorList.forEach(f -> {
            f.getNodes().forEach(n -> {
                System.out.println("n = " + n);
                n.getRelations().forEach(r -> {
                    System.out.println("r = " + r);
                    if(!relationList.contains(r)) {
                        relationList.add(r);
                    }
                });
                n.setXCoordinate(n.getXCoordinate());
                n.setYCoordinate(n.getYCoordinate());
            });
            InputStream is;
            try {
                File file = new File(f.getFloorImageExternalForm().substring("file:\\".length()));
                is = new FileInputStream(file.getAbsolutePath());
                if (file.exists()) {
                    byte[] baseArray = IOUtils.toByteArray(is);
                    is.close();
                    f.setBackgroundBase64URL("data:image/" + file.getAbsolutePath().split("\\.(?=[^\\.]+$)")[1] + ";base64," + Base64.encode(baseArray));
                }
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } catch (StringIndexOutOfBoundsException e) {
                System.out.println("No picture file saved!");
            }
            System.out.println(f.getBackgroundBase64URL());
        });


        System.out.println("Number of relations: " + relationList.size());
        //System.out.println("Number of Nodes: " + mapNodeList.size());
        System.out.println("Number of Floors: " + floorList.size());
    }

    public MapNode getCurrentNode() {
        return currentNode;
    }

    public void setCurrentNode(MapNode node) {
        if (currentNode != null) {
            currentNode.setEffect(null);
        }
        if (currentRelation != null) {
            currentRelation.setEffect(null);
        }
        if (node != null) {
            currentNode = null;
            currentNode = node;
            System.out.println("Current Node set! : " + currentNode);
            currentNode.setEffect(highlight);
        }
    }

    public ConnectRelation addRelation(MapNode firstNode, MapNode secondNode) {
        if (!duplicateRelation(firstNode, secondNode)) {
            System.out.println("----------\nOK\n----------");
            ConnectRelation newRelation = firstNode.connectToNode(secondNode);
            relationList.add(newRelation);


            session.save(firstNode);


            reloadFloor = true;

            System.out.println("EditorModel.addRelation");
            setChanged();
            notifyObservers();

            return newRelation;

        } else {
            System.out.println("----------\nNOT OK\n----------");
            Alert alert = new Alert(Alert.AlertType.ERROR);
            alert.setContentText("These 2 Nodes are already connected!");
            alert.setHeaderText("ERROR");
            alert.show();
            return null;
        }
    }

    public void removeRelation(ConnectRelation relation) {
        relationList.remove(relation);

        relation.getStartNode().getRelations().remove(relation);
        relation.getEndNode().getRelations().remove(relation);

        removeFromDb(relation);
        saveToDb(relation.getStartNode());
        saveToDb(relation.getEndNode());

        if (!batchRemove) {
            reloadFloor = true;
            setChanged();
            System.out.println("EditorModel.removeRelation");
            notifyObservers();
        }
    }

    public void removeRelation(ArrayList<ConnectRelation> relations) {
        batchRemove = true;
        relations.forEach(relation -> {
            removeRelation(relation);
        });
        setChanged();
        System.out.println("EditorModel.removeRelation");
        notifyObservers();
        batchRemove = false;
    }

    public void updateRelation(ConnectRelation relation) {
        saveToDb(relation);
    }

    public void addNode(MapNode node) {
        if (getCurrentFloor() == null)
            return;


        currentFloor.getNodes().add(node);
        node.setFloor(currentFloor);
        saveToDb(currentFloor);

        reloadFloor = true;

        setChanged();
        System.out.println("EditorModel.addNode");
        notifyObservers();
    }

    public void removeNode(MapNode mapNode) {
        if (mapNode.getRelations() != null) {
            removeRelation(new ArrayList<>(mapNode.getRelations()));
        }


        currentFloor.getNodes().remove(mapNode);

        removeFromDb(mapNode);
        saveToDb(currentFloor);

        reloadFloor = true;

        setChanged();
        System.out.println("EditorModel.removeNode");
        notifyObservers();
    }

    public void updateNode(MapNode mapNode) {
        saveToDb(mapNode);
        System.out.println("Updated mapNode = [" + mapNode + "]");
    }

    public Floor getCurrentFloor() {
        return currentFloor;
    }

    public void setCurrentFloor(Floor floor) {
        reloadFloor = true;
        this.currentFloor = floor;

        relationList.clear();

        if (getCurrentFloor().getNodes() != null) {
            getCurrentFloor().getNodes().forEach(n -> {
                n.getRelations().forEach(r -> {
                    if(!relationList.contains(r)){
                        relationList.add(r);
                    }
                });
            });
        }

        setChanged();
        System.out.println("EditorModel.setCurrentFloor");
        notifyObservers();
    }

    public ConnectRelation getCurrentRelation() {
        return currentRelation;
    }

    public void setCurrentRelation(ConnectRelation currentRelation) {
        if (this.currentNode != null) {
            this.currentNode.setEffect(null);
        }
        if (this.currentRelation != null) {
            this.currentRelation.setEffect(null);
        }
        this.currentRelation = currentRelation;
        this.currentRelation.setEffect(highlight);
    }

    public ObservableList<Floor> getFloorList() {
        return floorList;
    }

    public void setFloorList(ObservableList<Floor> floorList) {
        this.floorList = floorList;
    }

    public void addMapToCurrentFloor(String externalFormURL) {
        getCurrentFloor().setFloorImageExternalForm(externalFormURL);
        setChanged();
        System.out.println("EditorModel.addMapToCurrentFloor");
        notifyObservers();
        reloadFloor = true;
        saveToDb(getCurrentFloor());
    }

    public List<ConnectRelation> getRelationList() {
        return relationList;
    }


    private boolean duplicateRelation(MapNode startNode, MapNode endNode) {

        for (ConnectRelation r : startNode.getRelations()) {
            if ((r.getStartNode() == startNode && r.getEndNode() == endNode) || (r.getEndNode() == startNode && r.getStartNode() == endNode)) {
                return true;
            }
        }

        return false;
    }

    public boolean isReloadFloor() {
        return reloadFloor;
    }


    public boolean isReloadListView() {
        return reloadListView;
    }


    public void addFloor() {
        Floor floor = new Floor();
        floor.setName("New Floor");
        floor.setShortName("NF");
        floor.setFloorImageExternalForm("");
        //TODO Add to current building
        floorList.add(floor);
        setCurrentFloor(floor);

        saveToDb(floor);

        reloadFloor = true;
        reloadListView = true;

        setChanged();
        System.out.println("EditorModel.addFloor");
        notifyObservers();
    }

    private void addFloor(Floor newFloor) {
        //TODO Add to current building
        floorList.add(newFloor);
        setCurrentFloor(newFloor);

        new Thread(() -> {
            DatabaseRepository.getInstance().getSession().save(newFloor,-1);
        }).start();

        setCurrentFloor(newFloor);

        reloadFloor = true;
        reloadListView = true;

        System.out.println("EditorModel.addFloorAsNew");
        setChanged();
        notifyObservers();

    }

    public void removeFloor() {
        Floor tempFloor = currentFloor;
        new Thread(new RemoveFloorFromDbThread(tempFloor)).start();
        floorList.remove(currentFloor);
        if (floorList.isEmpty()) {
            relationList.clear();
        }
        currentFloor = null;
        reloadFloor = true;

        System.out.println("EditorModel.removeFloor");
        setChanged();
        notifyObservers();
    }

    public void updateFloor(Floor floor) {
        saveToDb(floor);
        reloadListView = true;
        System.out.println("EditorModel.updateFloor");
        setChanged();
        notifyObservers();
    }

    public List<MapNode> getLifts() {
        ArrayList<MapNode> liftNodes = new ArrayList<>();
        floorList.forEach(f -> {
            f.getNodes().forEach(n -> {
                if (n instanceof Lift) {
                    liftNodes.add(n);
                }
            });
        });
        return liftNodes;
    }

    public List<MapNode> getStairs() {
        ArrayList<MapNode> stairList = new ArrayList<>();
        floorList.forEach(f -> {
            f.getNodes().forEach(n -> {
                if (n instanceof Stair) {
                    stairList.add(n);
                }
            });
        });
        return stairList;
    }

    public void importFloorFromFile() throws IOException, JSONException {
        System.out.println("IMPORT FLOOR");

        FileChooser fileChooser = new FileChooser();
        fileChooser.getExtensionFilters().add(
                new FileChooser.ExtensionFilter(
                        "Wayfinding Editor Floor (*wfef)",
                        "*.wfef"
                ));

        File selectedFile = fileChooser.showOpenDialog(new Stage());

        String fileString = FileUtils.readFileToString(selectedFile);

        String [] splitString = fileString.split("-RELATIONS-");

        String floorString = splitString[0];
        String relationString = splitString[1];

        JSONObject floorJson = new JSONObject(floorString);
        JSONArray relationsJson = new JSONArray(relationString);

        Gson floorGson = new GsonBuilder().
                registerTypeAdapter(MapNode.class, new MapNodeDeserializer()).
                excludeFieldsWithoutExposeAnnotation().
                create();

        Gson relationsGson = new GsonBuilder().
                registerTypeAdapter(MapNode.class, new MapNodeDeserializer()).
                excludeFieldsWithoutExposeAnnotation().
                create();

        Floor newFloor = floorGson.fromJson(floorJson.toString(), Floor.class);
        List<ConnectRelation> relations = relationsGson.fromJson(relationsJson.toString(), new TypeToken<List<ConnectRelation>>(){}.getType());
        System.out.println(newFloor);

        boolean overwrite = false;
        Floor toDelete = new Floor();
        for (Floor floor : floorList) {
            if (floor.getName().equals(newFloor.getName())) {
                Alert alert = new Alert(Alert.AlertType.CONFIRMATION);
                alert.setTitle("");
                alert.setHeaderText("A Floor with the name \"" + newFloor.getName() + "\" already Exists");
                alert.setContentText("Would you like to overwrite the existing floor ?");

                ButtonType btYes = new ButtonType("Yes");
                ButtonType btNo = new ButtonType("No");
                ButtonType btBoth = new ButtonType("Keep Both");

                alert.getButtonTypes().setAll(btYes, btNo, btBoth);

                Optional<ButtonType> result = alert.showAndWait();

                if (result.get() == btYes) {
                    overwrite = true;
                    toDelete = floor;
                    break;
                } else if (result.get() == btNo) {
                    return;
                } else if (result.get() == btBoth) {
                    newFloor.setName(newFloor.getName() + " (1)");
                }
            }
        }

        if (overwrite) {
            new Thread(new RemoveFloorFromDbThread(toDelete)).start();
        }

        newFloor.setFloorId(null);
        newFloor.getNodes().forEach(n -> n.setFloor(newFloor));

        mapNodesToRelations(newFloor.getNodes(),relations);

        addFloor(newFloor);
    }

    /**
     * Imported relations from a JSON only have a reference to the NodeID of the MapNode.
     * This means that the GSON cannot directly map the relations to the correct MapNodes,
     * but instead generates new MapNodes with only a nodeId (Circular dependency is a general problem with JSON parsers).
     * We can use this nodeId to map the correct MapNodes to their relations.
     *
     * We can use the nodeID's in the relations to link the MapNodes after the import.
     * @param importedNodes
     *  The nodes which where imported from a file
     *
     * @param relations
     *  The relations which where imported from a file
     */
    private void mapNodesToRelations(List<MapNode> importedNodes, List<ConnectRelation> relations){
        int count = 0;
        for (ConnectRelation relation :
                relations) {
            for (MapNode node :
                    importedNodes) {
                if(relation.getStartNode().getNodeId() == node.getNodeId()){
                    relation.setStartNode(node);
                    node.getRelations().add(relation);
                    count++;
                }else if(relation.getEndNode().getNodeId() == node.getNodeId()){
                    relation.setEndNode(node);
                    node.getRelations().add(relation);
                    count++;
                }

                if(count == 2){
                    count = 0;
                    break;
                }
            }
        }
        importedNodes.forEach(n -> n.setNodeId(null));
        importedNodes.forEach(n -> {
            n.setCenterX(n.getXCoordinate());
            n.setCenterY(n.getYCoordinate());
        });
        relations.forEach(r -> r.setRelationId(null));
    }

    public void exportFloorToFile() throws IOException {
        System.out.println("EXPORT FLOOR");

        FileChooser fileChooser = new FileChooser();
        fileChooser.setTitle("Save");
        fileChooser.setInitialFileName(getCurrentFloor().getName());
        fileChooser.getExtensionFilters().add(
                new FileChooser.ExtensionFilter(
                        "Wayfinding Editor Floor",
                        "*.wfef"
                ));
        File fileToSave = fileChooser.showSaveDialog(new Stage());

        if (fileToSave == null) {
            return;
        }

        Gson gson = new GsonBuilder().
                excludeFieldsWithoutExposeAnnotation().
                setPrettyPrinting().
                registerTypeAdapter(ConnectRelation.class, new ConnectRelationSerializer()).
                create();

        String floorJson = gson.toJson(getCurrentFloor()) + "\n-RELATIONS-\n"+gson.toJson(getRelationList());

        FileUtils.writeStringToFile(fileToSave, floorJson);

    }

    public void generateHTML() {

        System.out.println("GENERATING HTML!!");

        Type nodeTypeObject = new TypeToken<ArrayList<MapNode>>() {
        }.getType();
        Type releationTypeObject = new TypeToken<ArrayList<ConnectRelation>>() {
        }.getType();
        Type floorTypeObject = new TypeToken<ArrayList<Floor>>() {
        }.getType();


        ArrayList<ConnectRelation> exportRelationList = new ArrayList<>();
        ArrayList<MapNode> mapNodeList = new ArrayList<>();

        getFloorList().forEach(f -> f.getNodes().forEach(n -> mapNodeList.add(n)));
        mapNodeList.forEach(n -> exportRelationList.addAll(n.getRelations()));


        /*relationList.forEach(r -> {
            if (!r.isTimeRestricted()) {
                r.setTimeTo("");
                r.setTimeFrom("");
            }
            if (!r.isDateRestricted()) {
                r.setDateTo("");
                r.setDateFrom("");
            }
            String[] time;
            if (!r.getTimeFrom().isEmpty()) {
                time = r.getTimeFrom().split(":");
                if (time[0].length() < 2) {
                    time[0] = "0" + time[0];
                }

                if (time[1].length() < 2) {
                    time[1] = "0" + time[0];
                }
                r.setTimeFrom(time[0] + ":" + time[1]);
            }

            if (!r.getTimeTo().isEmpty()) {
                time = r.getTimeTo().split(":");
                if (time[0].length() < 2) {
                    time[0] = "0" + time[0];
                }

                if (time[1].length() < 2) {
                    time[1] = "0" + time[0];
                }
                r.setTimeTo(time[0] + ":" + time[1]);
            }

        });*/

        Gson gson = new GsonBuilder().excludeFieldsWithoutExposeAnnotation().setPrettyPrinting().create();

        ArrayList<Floor> exportFloorList = new ArrayList<Floor>(floorList);

        //Check for Lifts and Stairs - Their Relations need to be added manually
        floorList.forEach(f -> {
            f.getNodes().forEach(n -> {
                if (n instanceof Lift) {
                    if (((Lift) n).getLiftAbove() != null) {
                        exportRelationList.add(new ConnectRelation(n, ((Lift) n).getLiftAbove()));
                    }
                }

                if (n instanceof Stair) {
                    if (((Stair) n).getStairAbove() != null) {
                        exportRelationList.add(new ConnectRelation(n, ((Stair) n).getStairAbove()));
                    }
                }
            });
        });

        String expfloor = gson.toJson(exportFloorList, floorTypeObject);
        String expRel = gson.toJson(exportRelationList, releationTypeObject);

        String js =
                "var floors = " + expfloor.toString() + ";" +
                        "var relations = " + expRel.toString() + ";";

        DirectoryChooser chooser = new DirectoryChooser();
        chooser.setTitle("Select generateHTML directory!");
        //File defaultDirectory = new File("C:\\");
        //chooser.setInitialDirectory(defaultDirectory);
        File selectedDirectory = chooser.showDialog(new Stage());

        Path p = Paths.get(selectedDirectory.getAbsolutePath() + "\\json.txt");
        System.out.println(expfloor + " \n " + expRel);
        byte data[] = js.getBytes();
        try {
            boolean fileExists = Files.exists(p);

            if (fileExists) {
                Files.delete(p);
                Files.createFile(p);
            } else {

                Files.createFile(p);
            }

            OutputStream out;
            out = new BufferedOutputStream(Files.newOutputStream(p, CREATE, APPEND));

            out.write(data, 0, data.length);

            out.flush();
            out.close();

        } catch (IOException ex) {
            System.out.println("Exception");
            Logger.getLogger(EditorModel.class.getName()).log(Level.SEVERE, null, ex);
        }
        System.out.println(p.toString());
        System.out.println(getCurrentFloor().getBackgroundBase64URL().length());

    }

    public void purgeDb() {
        session.purgeDatabase();
    }

    public void saveToDb(Object o) {
        Thread t = new Thread(new SaveToDbThread(o));
        t.setDaemon(true);
        t.start();
    }

    public void removeFromDb(Object o) {
        Thread t = new Thread(new RemoveFromDbThread(o));
        t.setDaemon(true);
        t.start();
    }

    public Timer getUpdateNodesTimer() {
        return updateNodesTimer;
    }
}
