/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.htl.leonding.digitalsignage.wayfinder.views.baseeditor;

import java.io.*;
import java.net.URL;
import java.util.*;

import javafx.application.Platform;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.Cursor;
import javafx.scene.Node;
import javafx.scene.Scene;

import javafx.scene.control.*;
import javafx.scene.control.Button;
import javafx.scene.input.KeyCode;
import javafx.scene.input.KeyEvent;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.Pane;

import org.htl.leonding.digitalsignage.wayfinder.entities.*;
import org.htl.leonding.digitalsignage.wayfinder.interfaces.IDoForThisSubtype;
import org.htl.leonding.digitalsignage.wayfinder.models.*;
import org.htl.leonding.digitalsignage.wayfinder.views.cornereditor.CornerEditorView;
import org.htl.leonding.digitalsignage.wayfinder.views.flooreditor.FloorEditorView;
import org.htl.leonding.digitalsignage.wayfinder.views.lifteditor.LiftEditorView;
import org.htl.leonding.digitalsignage.wayfinder.views.relationeditor.RelationEditorView;
import org.htl.leonding.digitalsignage.wayfinder.views.roomeditor.RoomEditorView;
import org.htl.leonding.digitalsignage.wayfinder.views.staireditor.StairEditorView;
import org.neo4j.ogm.json.JSONException;

import javax.inject.Inject;

/**
 * @author Cristopher
 */
public class BaseEditorPresenter implements Initializable, Observer {

    private Scene scene;

    @Inject
    private EditorModel model;

    @Inject
    private String nodeSize;
    @Inject
    private String relationWidth;

    @Inject
    private String text;

    @FXML
    private MenuItem miExportBuilding;
    @FXML
    private MenuItem miExportFloor;
    @FXML
    private MenuItem miImportBuilding;
    @FXML
    private MenuItem miImportFloor;
    @FXML
    private MenuItem miGenerateHTML;

    @FXML
    private Pane mapPane;
    @FXML
    private ChoiceBox<MapNodeType> cbMapNodeType;
    @FXML
    private Button btAddFloor;
    @FXML
    private ListView<Floor> lvFloors;
    @FXML
    private Pane propertyPane;
    @FXML
    private Button btRemoveFloor;
    @FXML
    private CheckBox cbShowLength;
    @FXML
    private Slider sliderNodeSize;
    @FXML
    private Slider sliderRelationWidth;

    @Override
    public void initialize(URL location, ResourceBundle resources) {
        text = "HELLO :D";
        Platform.setImplicitExit(false);
        model.addObserver(this);
        //DatabaseRepository.fillDb();
        initClickables();

        setMapHandlers();

        Platform.runLater(() -> {
            mapPane.getScene().setOnKeyPressed(onSceneKeyPressedHandler);
            mapPane.getScene().setOnKeyReleased(onSceneKeyReleasedHandler);
        });

        if (model.getCurrentFloor() != null) {
            drawFloor(model.getCurrentFloor());
        }
    }

    private void initClickables() {
        miImportFloor.setOnAction(event -> {
            try {
                model.importFloorFromFile();
            } catch (IOException e) {
                Alert alert = new Alert(Alert.AlertType.ERROR);
                alert.setContentText("Error opening File!");
                alert.show();
                e.printStackTrace();
            } catch (JSONException e) {
                Alert alert = new Alert(Alert.AlertType.ERROR);
                alert.setContentText("File is not in correct Format!");
                e.printStackTrace();
            }
        });

        miExportFloor.setOnAction(event -> {
            try {
                model.exportFloorToFile();
            } catch (IOException e) {
                Alert alert = new Alert(Alert.AlertType.ERROR);
                alert.setContentText("Erorr exporting File!");
                alert.show();
                e.printStackTrace();
            }
        });

        miGenerateHTML.setOnAction(event -> model.generateHTML());

        cbMapNodeType.getItems().setAll(MapNodeType.values());
        cbMapNodeType.getSelectionModel().selectFirst();

        lvFloors.setItems(model.getFloorList());

        lvFloors.getSelectionModel().selectedItemProperty().addListener((observable, oldValue, newValue) -> {
            if (newValue == null) {
                return;
            }
            model.setCurrentFloor(newValue);
            changeToFloorProperty();
        });

        lvFloors.setOnMouseClicked(event -> changeToFloorProperty());

        lvFloors.getSelectionModel().select(0);

        btAddFloor.setOnAction(event -> {
            model.addFloor();
            changeToFloorProperty();
        });
        btRemoveFloor.setOnAction(event -> {
            model.removeFloor();
        });

        cbShowLength.selectedProperty().addListener((observable, oldValue, newValue) -> {
            drawFloor(model.getCurrentFloor());
        });

        sliderNodeSize.setMin(3);
        sliderNodeSize.setMax(20);

        sliderRelationWidth.setMin(3);
        sliderRelationWidth.setMax(10);

        sliderNodeSize.valueProperty().addListener((observable, oldValue, newValue) -> {
            System.out.println("--- NODE SLIDER CHANGED ---");
            this.nodeSize = newValue.toString();
            model.getFloorList().forEach(f ->
                    f.getNodes().forEach(n -> {
                        n.setRadius(newValue.doubleValue());
                    }));
        });

        sliderNodeSize.onDragDoneProperty().addListener((observable1, oldValue1, newValue1) -> {
            System.out.println("NODE SIZE SLIDER CLICKED");
        });

        sliderRelationWidth.valueProperty().addListener((observable, oldValue, newValue) -> {
            System.out.println("--- RELATiON SLIDER CHANGED");
            this.relationWidth = newValue.toString();
            model.getFloorList().forEach(f ->
                    f.getNodes().forEach(n -> {
                        n.getRelations().forEach(r -> {
                            r.setStrokeWidth(newValue.doubleValue());
                        });
                    }));
        });

        sliderRelationWidth.onDragDoneProperty().addListener((observable, oldValue, newValue) -> {
            System.out.println("RELATION SLIDER CLICKED");
        });

        sliderNodeSize.setValue(Double.valueOf(nodeSize));
        sliderRelationWidth.setValue(Double.valueOf(relationWidth));

    }

    private void drawFloor(Floor floor) {
        if (model.getCurrentFloor() == null)
            return;

        try {
            mapPane.getChildren().clear();
        } catch (Exception e) {
            mapPane.getChildren().clear();
        }

        System.out.println("CLEAR");

        if (model.getCurrentFloor().getNodes() != null) {
            model.getCurrentFloor().getNodes().forEach(n -> {
                n.setRadius(Double.valueOf(nodeSize));
                System.out.println("n = " + n);
                if (!mapPane.getChildren().contains(n)) {
                    mapPane.getChildren().add(n);
                }
                setNodeHandlers(n);
            });
        }

        for (ConnectRelation r : model.getRelationList()) {
            setRelationHandlers(r);
            r.setStrokeWidth(Double.valueOf(relationWidth));
            r.updateStartAndEndAndText(model.getCurrentFloor().getFloorWidth(), model.getCurrentFloor().getFloorHeight());
            System.out.println("r = " + r);
            if (!mapPane.getChildren().contains(r)) {
                mapPane.getChildren().add(r);
            }
            if (cbShowLength.isSelected()) {
                mapPane.getChildren().add(r.getText());
            }
            r.toBack();
        }
        setMapPaneBackground(model.getCurrentFloor().getFloorImageExternalForm());
    }

    private void redrawBackground() {
        setMapPaneBackground(model.getCurrentFloor().getFloorImageExternalForm());
    }

    //<baseeditor-fold desc="--- Node - Events ---">
    double orgSceneX;
    double orgSceneY;
    double orgTranslateX;
    double orgTranslateY;

    boolean dragged = false;

    private void setNodeHandlers(Node node) {
        node.setOnMousePressed(onNodePressedHandler);
        node.setOnMouseDragged(onNodeDraggedHandler);
        node.setOnMouseEntered(onNodeHoverEntered);
        node.setOnMouseExited(onNodeHoverExited);
        node.setOnMouseClicked(onNodeClickedHandler);
    }

    EventHandler<MouseEvent> onNodeClickedHandler = new EventHandler<MouseEvent>() {
        @Override
        public void handle(MouseEvent event) {
            MapNode targetNode = (MapNode) event.getTarget();
            System.out.println("CLICKED --- " + targetNode);
            if (event.isAltDown()) {
                model.removeNode(targetNode);
                model.setCurrentNode(null);
                return;
            } else if (shiftPressed) {
                if (firstPressedNode != null && targetNode != null && targetNode != firstPressedNode) {
                    ConnectRelation newRelation;
                    if ((newRelation = model.addRelation(firstPressedNode, targetNode)) != null) {
                        setRelationHandlers(newRelation);
                        model.setCurrentRelation(newRelation);
                        changeToRelationProperty();
                        dragged = false;
                        firstPressedNode = targetNode;
                        return;
                    } else {
                        firstPressedNode = null;
                        shiftPressed = false;
                    }
                } else if (event.getTarget() instanceof MapNode) {
                    firstPressedNode = (MapNode) event.getTarget();
                }
                return;
            } else if (dragged) {
                System.out.println("DRAGGED");
                dragged = false;
            }
            model.updateNode(targetNode);
            System.out.println(model.getCurrentNode());
            System.out.println(event.getTarget());
            ((IDoForThisSubtype) model.getCurrentNode()).setPropertyForThisClass(BaseEditorPresenter.this);
        }
    };

    EventHandler<MouseEvent> onNodeHoverEntered = new EventHandler<MouseEvent>() {
        @Override
        public void handle(MouseEvent event) {
            mapPane.getScene().setCursor(Cursor.HAND);
        }
    };

    EventHandler<MouseEvent> onNodeHoverExited = new EventHandler<MouseEvent>() {
        @Override
        public void handle(MouseEvent event) {
            mapPane.getScene().setCursor(Cursor.DEFAULT);
        }

    };

    EventHandler<MouseEvent> onNodePressedHandler = new EventHandler<MouseEvent>() {
        @Override
        public void handle(MouseEvent t) {
            MapNode targetNode = (MapNode) t.getSource();
            System.out.println("PRESSED ----  " + targetNode);
            model.setCurrentNode(targetNode);
            ((IDoForThisSubtype) model.getCurrentNode()).setPropertyForThisClass(BaseEditorPresenter.this);

            orgSceneX = t.getSceneX();
            orgSceneY = t.getSceneY();

            orgTranslateX = targetNode.getCenterX();
            orgTranslateY = targetNode.getCenterY();

        }
    };

    EventHandler<MouseEvent> onNodeDraggedHandler = new EventHandler<MouseEvent>() {
        @Override
        public void handle(MouseEvent t) {
            dragged = true;

            MapNode currentNode = (MapNode) t.getSource();

            if (t.isAltDown()) {
                return;
            } else if (t.isShiftDown())
                return;

            double offsetX = t.getSceneX() - orgSceneX;
            double offsetY = t.getSceneY() - orgSceneY;

            double newTranslateX = orgTranslateX + offsetX;
            double newTranslateY = orgTranslateY + offsetY;

            currentNode.setXCoordinate(newTranslateX);
            currentNode.setYCoordinate(newTranslateY);
            currentNode.notifyObservers();

            System.out.println("DRAGGED ---  " + currentNode);
        }
    };
    //</baseeditor-fold>


    //<baseeditor-fold desc="--- Map - Events ---">
    private void setMapHandlers() {
        this.mapPane.setOnMouseClicked(onMapPaneClickedHandler);
    }

    EventHandler<MouseEvent> onMapPaneClickedHandler = new EventHandler<MouseEvent>() {
        @Override
        public void handle(MouseEvent event) {

            if (model.getCurrentFloor() == null)
                return;

            if (!event.isControlDown())
                return;

            double radius = Double.valueOf(BaseEditorPresenter.this.nodeSize);
            MapNode node;
            switch (cbMapNodeType.getSelectionModel().getSelectedItem().toString()) {
                case "Room":
                    node = new Room(radius);
                    break;
                case "Corner":
                    node = new Corner(radius);
                    break;
                case "Stair":
                    node = new Stair(radius);
                    break;
                case "Lift":
                    node = new Lift(radius);
                    break;
                default:
                    return;
            }
            node.setXCoordinate(event.getX());
            node.setYCoordinate(event.getY());
            setNodeHandlers(node);
            model.addNode(node);
            model.setCurrentNode(node);
            ((IDoForThisSubtype) node).setPropertyForThisClass(BaseEditorPresenter.this);
        }
    };
    //</baseeditor-fold>


    //<baseeditor-fold desc="--- Create Relation Events ---">
    boolean shiftPressed;
    MapNode firstPressedNode = null;

    EventHandler<KeyEvent> onSceneKeyPressedHandler = new EventHandler<KeyEvent>() {
        @Override
        public void handle(KeyEvent event) {
            if (event.getCode() == KeyCode.SHIFT) {
                shiftPressed = true;
            } else if (event.getCode() == KeyCode.DIGIT1) {
                cbMapNodeType.getSelectionModel().select(0);
            } else if (event.getCode() == KeyCode.DIGIT2) {
                cbMapNodeType.getSelectionModel().select(1);
            } else if (event.getCode() == KeyCode.DIGIT3) {
                cbMapNodeType.getSelectionModel().select(2);
            } else if (event.getCode() == KeyCode.DIGIT4) {
                cbMapNodeType.getSelectionModel().select(3);
            } else if (event.getCode() == KeyCode.F1) {
                Alert helpAlert = new Alert(Alert.AlertType.INFORMATION);
                helpAlert.setHeaderText("Help");
                helpAlert.setContentText(
                        "STRG + CLICK - Place Node\n\n" +
                                "SHIFT + CLICK (2 Nodes) - Connect\n\n" +
                                "ALT + CLICK - Delete Node or Connection\n\n" +
                                "1,2,3,4 - Switch between Room, Corner, Stair, Lift"
                );
                helpAlert.show();
            }
        }
    };
    EventHandler<KeyEvent> onSceneKeyReleasedHandler = new EventHandler<KeyEvent>() {
        @Override
        public void handle(KeyEvent event) {
            if (event.getCode() == KeyCode.SHIFT) {
                shiftPressed = false;
                firstPressedNode = null;
            }
        }
    };
    //</baseeditor-fold>


    //<baseeditor-fold desc="--- Relation - Events ---">
    private void setRelationHandlers(ConnectRelation relation) {
        relation.setOnMouseClicked(onRelationClickHandler);
        relation.setOnMouseEntered(onRelationHoverEnteredHandler);
        relation.setOnMouseExited(onRelationHoverExitedHandler);
    }

    EventHandler<MouseEvent> onRelationClickHandler = new EventHandler<MouseEvent>() {
        @Override
        public void handle(MouseEvent event) {
            if (event.isAltDown()) {
                model.removeRelation((ConnectRelation) event.getTarget());
                return;
            }
            model.setCurrentRelation((ConnectRelation) event.getTarget());
            changeToRelationProperty();
        }
    };

    EventHandler<MouseEvent> onRelationHoverEnteredHandler = new EventHandler<MouseEvent>() {
        @Override
        public void handle(MouseEvent event) {
            mapPane.getScene().setCursor(Cursor.HAND);
        }
    };

    EventHandler<MouseEvent> onRelationHoverExitedHandler = new EventHandler<MouseEvent>() {
        @Override
        public void handle(MouseEvent event) {
            mapPane.getScene().setCursor(Cursor.DEFAULT);
        }
    };
    //</baseeditor-fold>


    //<baseeditor-fold desc="--- Change PropertyPane Methods ---">
    public void changeToFloorProperty() {
        propertyPane.getChildren().clear();
        propertyPane.getChildren().add(new FloorEditorView().getView());
        System.out.println("FLOOR--");
    }

    public void changeToRelationProperty() {
        propertyPane.getChildren().clear();
        propertyPane.getChildren().add(new RelationEditorView().getView());
        System.out.println("Relation");
    }

    public void changeToRoomProperty() {
        propertyPane.getChildren().clear();
        propertyPane.getChildren().add(new RoomEditorView().getView());
        System.out.println("Room");
    }

    public void changeToCornerProperty() {
        propertyPane.getChildren().clear();
        propertyPane.getChildren().add(new CornerEditorView().getView());
        System.out.println("Corner");
    }

    public void changeToStairProperty() {
        propertyPane.getChildren().clear();
        propertyPane.getChildren().add(new StairEditorView().getView());
        System.out.println("Stair");
    }

    public void changeToLiftProperty() {
        propertyPane.getChildren().clear();
        propertyPane.getChildren().add(new LiftEditorView().getView());
        System.out.println("Lift");
    }

    //</baseeditor-fold>


    //<baseeditor-fold desc="--- Room Property Listeners ---">
    //</baseeditor-fold>


    //<baseeditor-fold desc="--- Relation Property Listeners ---">
    //</baseeditor-fold>

    private void setMapPaneBackground(String imageExternalUrl) {
        mapPane.setStyle(
                "-fx-background-image: url('" + imageExternalUrl + "'); " +
                        "-fx-background-size: stretch;" +
                        "-fx-background-position: 0 0; " +
                        "-fx-background-repeat: no-repeat;"
        );
    }

    private void removeMapPaneBackground() {
        mapPane.setStyle(
                "-fx-background-image: none;" +
                        "-fx-background-size: stretch;" +
                        "-fx-background-position: 0 0; " +
                        "-fx-background-repeat: no-repeat;"
        );
    }

    @Override
    public void update(Observable o, Object arg) {

        if (model.getCurrentFloor() == null) {
            if (model.getFloorList().size() == 0 || model.getFloorList() == null) {
                removeMapPaneBackground();
                mapPane.getChildren().clear();
                propertyPane.getChildren().clear();

            } else {
                model.setCurrentFloor(model.getFloorList().get(0));
            }
            return;
        }

        if (model.isReloadFloor()) {
            if (model.getCurrentFloor() != null) {
                System.out.println("DRAWFLOOR");
                drawFloor(model.getCurrentFloor());
            }
        }

        if (model.isReloadListView()) {
            Platform.runLater(() -> {
                lvFloors.setItems(null);
                lvFloors.setItems(model.getFloorList());
                lvFloors.getSelectionModel().select(model.getCurrentFloor());
            });
        }
        model.resetUpdaters();
        redrawBackground();
    }
}
