package org.htl.leonding.digitalsignage.wayfinder.views.flooreditor;

import com.sun.org.apache.xerces.internal.impl.dv.util.Base64;
import javafx.application.Platform;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.Alert;
import javafx.scene.control.Button;
import javafx.scene.control.TextField;
import javafx.stage.FileChooser;
import javafx.stage.Stage;
import org.apache.commons.io.IOUtils;
import org.htl.leonding.digitalsignage.wayfinder.entities.Floor;
import org.htl.leonding.digitalsignage.wayfinder.models.EditorModel;

import javax.inject.Inject;
import java.io.*;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ResourceBundle;

/**
 * Created by Cristopher on 03/01/2016.
 */
public class FloorEditorPresenter implements Initializable {

    @Inject
    EditorModel model;

    @FXML
    Button btAddPicture;

    @FXML
    TextField tfFloorURL;
    @FXML
    TextField tfFloorName;
    @FXML
    TextField tfShortName;
    @FXML
    TextField tfWidth;
    @FXML
    TextField tfHeight;
    @FXML
    Button btSave;

    @Override
    public void initialize(URL location, ResourceBundle resources) {
        initFields();
        updateUI();
    }

    public void initFields() {
        btAddPicture.setOnAction(event -> {
            Stage fileStage = new Stage();

            FileChooser fileChooser = new FileChooser();
            fileChooser.setTitle("Choose a floor plan");
            fileChooser.getExtensionFilters().addAll(
                    new FileChooser.ExtensionFilter("Image Files", "*.png", "*.jpg", "*.gif"));

            File selectedFile = fileChooser.showOpenDialog(fileStage);


            try {
                FileInputStream is = new FileInputStream(selectedFile.getAbsolutePath());
                Platform.runLater(() -> {
                    byte[] baseArray = new byte[0];
                    try {
                        baseArray = IOUtils.toByteArray(is);
                        model.getCurrentFloor().setBackgroundBase64URL("data:image/" + selectedFile.getAbsolutePath().split("\\.(?=[^\\.]+$)")[1] + ";base64," + Base64.encode(baseArray));
                        is.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                });
            } catch (FileNotFoundException e) {
                System.out.println("File not found");
            } catch (NullPointerException e){
                System.out.println("NullPointer exception while choosing Floor");
            }

            try {
                URL fileUrl = selectedFile.toURI().toURL();
                model.addMapToCurrentFloor(fileUrl.toExternalForm());
            } catch (MalformedURLException e) {
                e.printStackTrace();
            } catch (NullPointerException e) {
                e.printStackTrace();
            }
            updateUI();
        });

        tfFloorName.focusedProperty().addListener((observable, oldValue, newValue) -> {
            if (!newValue) {
                if (!model.getCurrentFloor().getName().equals(tfFloorName.getText())) {
                    for(Floor floor : model.getFloorList()){
                        if(floor.getName().equals(tfFloorName.getText())){
                            Alert alert = new Alert(Alert.AlertType.ERROR);
                            alert.setContentText("A floor with this name already exists!");
                            alert.show();
                            tfFloorName.setText("");
                            return;
                        }
                    }
                    model.getCurrentFloor().setName(tfFloorName.getText());
                    model.updateFloor(model.getCurrentFloor());
                }
            }
        });
        tfShortName.focusedProperty().addListener((observable, oldValue, newValue) -> {
            if (!newValue) {
                if (!model.getCurrentFloor().getShortName().equals(tfShortName.getText())) {
                    model.getCurrentFloor().setShortName(tfShortName.getText());
                    model.updateFloor(model.getCurrentFloor());
                }
            }
        });

        tfWidth.focusedProperty().addListener((observable, oldValue, newValue) -> {
            if(!newValue){
                if(isDouble(tfWidth.getText())){
                    model.getCurrentFloor().setFloorWidth(Double.parseDouble(tfWidth.getText()));
                }else{
                    model.getCurrentFloor().setFloorWidth(0);
                }
                model.saveToDb(model.getCurrentFloor());
            }
        });

        tfHeight.focusedProperty().addListener((observable, oldValue, newValue) -> {
            if(!newValue){
                if(isDouble(tfHeight.getText())){
                    model.getCurrentFloor().setFloorHeight(Double.parseDouble(tfHeight.getText()));
                }else{
                    model.getCurrentFloor().setFloorHeight(0);
                }
                model.saveToDb(model.getCurrentFloor());
            }
        });

        btSave.onActionProperty().addListener((observable, oldValue, newValue) -> {
            updateUI();
        });
    }

    private void updateUI() {
        if (model.getCurrentFloor() != null) {
            tfFloorName.setText(model.getCurrentFloor().getName());
            tfShortName.setText(model.getCurrentFloor().getShortName());
            tfFloorURL.setText(model.getCurrentFloor().getFloorImageExternalForm());
            tfWidth.setText(String.valueOf(model.getCurrentFloor().getFloorWidth()));
            tfHeight.setText(String.valueOf(model.getCurrentFloor().getFloorHeight()));
        }
    }

    private boolean isDouble(String value) {
        try {
            Double.parseDouble(value);
        } catch (NumberFormatException e) {
            return false;
        }
        return true;
    }

}
