package org.htl.leonding.digitalsignage.wayfinder.views.lifteditor;

import javafx.application.Platform;
import javafx.collections.FXCollections;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.Button;
import javafx.scene.control.ChoiceBox;
import javafx.scene.control.TextField;
import org.htl.leonding.digitalsignage.wayfinder.entities.Lift;
import org.htl.leonding.digitalsignage.wayfinder.entities.MapNode;
import org.htl.leonding.digitalsignage.wayfinder.models.EditorModel;

import javax.inject.Inject;
import java.net.URL;
import java.util.*;

/**
 * Created by Cristopher on 03/01/2016.
 */
public class LiftEditorPresenter implements Initializable {

    @Inject
    EditorModel model;

    @FXML
    private TextField tfLiftName;

    @FXML
    private ChoiceBox liftAbove;

    @FXML
    private ChoiceBox liftBelow;

    @FXML
    private Button btRemoveLiftAbove;

    @FXML
    private Button btRemoveLiftBelow;

    Lift currentLift;

    @Override
    public void initialize(URL location, ResourceBundle resources) {
        currentLift = (Lift) model.getCurrentNode();

        System.out.println(currentLift);

        Platform.runLater(() -> {
            tfLiftName.setText(currentLift.getName());
            List<MapNode> lifts = model.getLifts();
            lifts.remove(currentLift);
            liftAbove.setItems(FXCollections.observableList(model.getLifts()));
            liftBelow.setItems(FXCollections.observableList(model.getLifts()));
        });

        tfLiftName.focusedProperty().addListener((observable, oldValue, newValue) -> {
            if (!newValue) {
                Lift lift = (Lift) model.getCurrentNode();
                lift.setName(tfLiftName.getText());
                model.updateNode(lift);
            }
        });

        liftAbove.getSelectionModel().selectedItemProperty().addListener((observable, oldValue, newValue) -> {
            System.out.println("SELECTED");
            currentLift.setLiftAbove((Lift) newValue);
            model.updateNode(currentLift);
        });

        liftBelow.getSelectionModel().selectedItemProperty().addListener((observable, oldValue, newValue) -> {
            System.out.println("SELECTED");
            currentLift.setLiftBelow((Lift) newValue);
            model.updateNode(currentLift);
        });

        btRemoveLiftAbove.setOnAction(event -> {
            currentLift.setLiftAbove(null);
            updateUI();
        });

        btRemoveLiftBelow.setOnAction(event -> {
            currentLift.setLiftBelow(null);
            updateUI();
        });


        updateUI();
    }

    public void updateUI() {
        liftAbove.getSelectionModel().select(currentLift.getLiftAbove());
        liftBelow.getSelectionModel().select(currentLift.getLiftBelow());
    }
}
