package org.htl.leonding.digitalsignage.wayfinder.views.relationeditor;

import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.Button;
import javafx.scene.control.CheckBox;
import javafx.scene.control.DatePicker;
import javafx.scene.control.TextField;
import javafx.scene.text.Text;
import jfxtras.scene.control.CalendarTimePicker;
import org.htl.leonding.digitalsignage.wayfinder.entities.ConnectRelation;
import org.htl.leonding.digitalsignage.wayfinder.entities.MapNode;
import org.htl.leonding.digitalsignage.wayfinder.models.EditorModel;

import javax.inject.Inject;
import java.net.URL;
import java.text.DecimalFormat;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.Calendar;
import java.util.ResourceBundle;

/**
 * Created by Cristopher on 03/01/2016.
 */
public class RelationEditorPresenter implements Initializable {

    @Inject
    EditorModel model;

    @FXML
    private TextField tfLength;

    @FXML
    private CheckBox cbRestrictTime;

    @FXML
    private CheckBox cbRestrictDate;

    @FXML
    private CalendarTimePicker tpTimeFrom;

    @FXML
    private CalendarTimePicker tpTimeTo;

    @FXML
    private DatePicker dpDateFrom;

    @FXML
    private DatePicker dpDateTo;

    @FXML
    private CheckBox cbBiDirectional;

    @FXML
    private CheckBox cbDisabledFriendly;

    @FXML
    private Text textStartNode;

    @FXML
    private Text textEndNode;

    @FXML
    private Button btSwitch;

    @FXML
    private CheckBox cbAutoLength;

    private ConnectRelation relation;

    @Override
    public void initialize(URL location, ResourceBundle resources) {
        relation = model.getCurrentRelation();

        tpTimeFrom.setDisable(relation.isTimeRestricted());
        tpTimeTo.setDisable(relation.isTimeRestricted());

        dpDateFrom.setDisable(relation.isDateRestricted());
        dpDateTo.setDisable(relation.isDateRestricted());

        btSwitch.setText("-->");

        btSwitch.setOnAction(event1 -> {
            MapNode helper;

            helper = relation.getStartNode();
            relation.setStartNode(relation.getEndNode());
            relation.setEndNode(helper);

            model.updateRelation(relation);

            updateUI();
        });

        cbDisabledFriendly.selectedProperty().addListener((observable3, oldValue3, newValue3) -> {
            if (cbDisabledFriendly.isSelected()) {
                relation.setDisabledFriendly(true);
                model.updateRelation(relation);
            } else {
                relation.setDisabledFriendly(false);
                model.updateRelation(relation);
            }
        });

        cbBiDirectional.selectedProperty().addListener((observable2, oldValue2, newValue2) -> {
            if (cbBiDirectional.isSelected()) {
                relation.setBidirectional(true);
                model.updateRelation(relation);
            } else {
                relation.setBidirectional(false);
                model.updateRelation(relation);
            }
        });

        tfLength.focusedProperty().addListener((observable1, oldValue1, newValue1) -> {
            if (!newValue1) {
                if (isDouble(tfLength.getText())) {
                    model.getCurrentRelation().setLength(Double.parseDouble(tfLength.getText()));
                    model.updateRelation(model.getCurrentRelation());
                } else {
                    model.getCurrentRelation().setLength(0);
                    model.updateRelation(model.getCurrentRelation());
                }
            }
        });

        cbRestrictTime.selectedProperty().addListener((observable, oldValue, newValue) -> {
            if (cbRestrictTime.isSelected()) {
                tpTimeTo.setDisable(false);
                tpTimeFrom.setDisable(false);
                relation.setTimeRestricted(true);
                model.updateRelation(relation);

            } else {
                tpTimeTo.setDisable(true);
                tpTimeFrom.setDisable(true);
                relation.setTimeRestricted(false);
                model.updateRelation(relation);
            }
        });

        cbRestrictDate.selectedProperty().addListener((observable, oldValue, newValue) -> {
            if (cbRestrictDate.isSelected()) {
                dpDateTo.setDisable(false);
                dpDateFrom.setDisable(false);
                relation.setDateRestricted(true);
                model.updateRelation(relation);
            } else {
                dpDateTo.setDisable(true);
                dpDateFrom.setDisable(true);
                relation.setTimeRestricted(false);
                model.updateRelation(relation);
            }
        });

        tpTimeTo.setOnMouseExited(event -> {
            String hour;
            String minute;

            hour = String.valueOf(tpTimeTo.getCalendar().get(Calendar.HOUR_OF_DAY));
            minute = String.valueOf(tpTimeTo.getCalendar().get(Calendar.MINUTE));

            String time = hour + ":" + minute;

            relation.setTimeTo(time);
            model.updateRelation(relation);
        });

        tpTimeFrom.setOnMouseExited(event -> {
            String hour;
            String minute;

            hour = String.valueOf(tpTimeFrom.getCalendar().get(Calendar.HOUR_OF_DAY));
            minute = String.valueOf(tpTimeFrom.getCalendar().get(Calendar.MINUTE));

            String time = hour + ":" + minute;

            relation.setTimeFrom(time);
            model.updateRelation(relation);
        });

        dpDateFrom.focusedProperty().addListener((observable, oldValue, newValue) -> {
            if (!newValue) {
                relation.setDateFrom(dpDateFrom.getValue().format(DateTimeFormatter.ofPattern("dd.MM.yyyy")).toString());
                model.updateRelation(relation);
            }
        });
        dpDateTo.focusedProperty().addListener((observable, oldValue, newValue) -> {
            if (!newValue) {
                relation.setDateTo(dpDateTo.getValue().format(DateTimeFormatter.ofPattern("dd.MM.yyyy")).toString());
                model.updateRelation(relation);
            }
        });

        cbAutoLength.selectedProperty().addListener((observable, oldValue, newValue) -> {
            if(newValue){
                relation.setAutomaticLength(true);
                relation.updateStartAndEndAndText(
                        relation.getStartNode().getFloor().getFloorWidth(),
                        relation.getStartNode().getFloor().getFloorHeight()
                );
            }else{
                relation.setAutomaticLength(false);
            }
            model.updateRelation(relation);
            updateUI();
        });

        updateUI();

    }

    private void updateUI() {
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("dd.MM.yyyy");

        ConnectRelation relation = model.getCurrentRelation();

        cbAutoLength.setSelected(relation.isAutomaticLength());

        cbRestrictDate.setSelected(relation.isDateRestricted());
        cbRestrictTime.setSelected(relation.isTimeRestricted());

        cbBiDirectional.setSelected(relation.isBidirectional());

        cbDisabledFriendly.setSelected(relation.isDisabledFriendly());

        tpTimeFrom.setDisable(!relation.isTimeRestricted());
        tpTimeTo.setDisable(!relation.isTimeRestricted());

        dpDateFrom.setDisable(!relation.isDateRestricted());
        dpDateTo.setDisable(!relation.isDateRestricted());

        tfLength.setText(new DecimalFormat("#0.0").format(relation.getLength()).toString());

        if (!relation.getTimeTo().isEmpty() && !relation.getTimeFrom().isEmpty()) {
            tpTimeFrom.getCalendar().set(Calendar.HOUR_OF_DAY, Integer.valueOf((relation.getTimeFrom().split(":")[0]).trim()));
            tpTimeFrom.getCalendar().set(Calendar.MINUTE, Integer.valueOf((relation.getTimeFrom().split(":")[1]).trim()));


            tpTimeTo.getCalendar().set(Calendar.HOUR_OF_DAY, Integer.valueOf((relation.getTimeTo().split(":")[0]).trim()));
            tpTimeTo.getCalendar().set(Calendar.MINUTE, Integer.valueOf((relation.getTimeTo().split(":")[1]).trim()));
        }

        if (!relation.getDateFrom().isEmpty() && !relation.getDateTo().isEmpty()) {
            dpDateTo.setValue(LocalDate.parse(relation.getDateTo(), formatter));
            dpDateFrom.setValue(LocalDate.parse(relation.getDateFrom(), formatter));
        }

        if (relation.getStartNode().getName() == null || relation.getStartNode().getName().isEmpty()) {
            textStartNode.setText(relation.getStartNode().getNodeId().toString());
        } else {
            textStartNode.setText(relation.getStartNode().getName());
        }

        if (relation.getEndNode().getName() == null || relation.getEndNode().getName().isEmpty()) {
            textEndNode.setText(relation.getEndNode().getNodeId().toString());
        } else {
            textEndNode.setText(relation.getEndNode().getName());
        }
    }

    private boolean isDouble(String value) {
        try {
            Double.parseDouble(value);
        } catch (NumberFormatException e) {
            return false;
        }
        return true;
    }
}
