package org.htl.leonding.digitalsignage.wayfinder.views.staireditor;

import javafx.application.Platform;
import javafx.collections.FXCollections;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.Button;
import javafx.scene.control.ChoiceBox;
import javafx.scene.control.TextField;
import org.htl.leonding.digitalsignage.wayfinder.entities.MapNode;
import org.htl.leonding.digitalsignage.wayfinder.entities.Stair;
import org.htl.leonding.digitalsignage.wayfinder.models.EditorModel;

import javax.inject.Inject;
import java.net.URL;
import java.util.*;

/**
 * Created by Cristopher on 03/01/2016.
 */
public class StairEditorPresenter implements Initializable {

    @Inject
    EditorModel model;

    @FXML
    private TextField tfStairName;

    @FXML
    private Button btRemoveStairAbove;

    @FXML
    private Button btRemoveStairBelow;

    @FXML
    private ChoiceBox stairBelow;

    @FXML
    private ChoiceBox stairAbove;

    Stair currentStair;

    @Override
    public void initialize(URL location, ResourceBundle resources) {
        currentStair = (Stair) model.getCurrentNode();
        System.out.println(currentStair.getNodeId());

        Platform.runLater(() -> {
            tfStairName.setText(currentStair.getName());
            List<MapNode> stairs = model.getStairs();
            stairs.remove(currentStair);
            stairAbove.setItems(FXCollections.observableList(stairs));
            stairBelow.setItems(FXCollections.observableList(stairs));
        });

        tfStairName.focusedProperty().addListener((observable, oldValue, newValue) -> {
            if (!newValue) {
                currentStair.setName(tfStairName.getText());
                model.updateNode(currentStair);
            }
        });

        stairBelow.getSelectionModel().selectedItemProperty().addListener((observable, oldValue, newValue) -> {
            System.out.println("SELECTED ");
            currentStair.setStairBelow((Stair) newValue);
            model.updateNode((MapNode) currentStair);
        });

        stairAbove.getSelectionModel().selectedItemProperty().addListener((observable, oldValue, newValue) -> {
            System.out.println("SELECTED ");
            currentStair.setStairAbove((Stair) newValue);
            model.updateNode((MapNode) currentStair);
        });

        btRemoveStairBelow.setOnAction(event -> {
            System.out.println("REMOVED");
            currentStair.setStairBelow(null);
            model.updateNode(currentStair);
        });

        btRemoveStairAbove.setOnAction(event -> {
            System.out.println("REMOVED");
            currentStair.setStairAbove(null);
            model.updateNode(currentStair);
        });

        updateUI();
    }

    private void updateUI() {
        Platform.runLater(() -> {
            stairAbove.getSelectionModel().select(currentStair.getStairAbove());
            stairBelow.getSelectionModel().select(currentStair.getStairBelow());
        });
    }
}
